--- old/usr/src/uts/i86pc/os/cpuid.c	Mon Jan 13 09:14:47 2020
+++ new/usr/src/uts/i86pc/os/cpuid.c	Mon Jan 13 09:14:47 2020
@@ -23,6 +23,7 @@
  * Copyright (c) 2011, 2016 by Delphix. All rights reserved.
  * Copyright 2013 Nexenta Systems, Inc. All rights reserved.
  * Copyright 2014 Josef "Jeff" Sipek <jeffpc@josefsipek.net>
+ * Copyright 2020 Joyent, Inc.
  */
 /*
  * Copyright (c) 2010, Intel Corporation.
@@ -31,9 +32,6 @@
 /*
  * Portions Copyright 2009 Advanced Micro Devices, Inc.
  */
-/*
- * Copyright 2020 Joyent, Inc.
- */
 
 /*
  * CPU Identification logic
@@ -1227,7 +1225,9 @@
  * Whenever we hit cpuid_scan_security() on the boot CPU and we're still on pass
  * 1 of the cpuid logic, then we can completely turn off TSX. Notably this
  * should happen twice. Once in the normal cpuid_pass1() code and then a second
- * time after we do the initial microcode update.
+ * time after we do the initial microcode update.  As a result we need to be
+ * careful in cpuid_apply_tsx() to only use the MSR if we've loaded a suitable
+ * microcode on the current CPU (which happens prior to cpuid_pass_ucode()).
  *
  * If TAA has been fixed, then it will be enumerated in IA32_ARCH_CAPABILITIES
  * as TAA_NO. In such a case, we will still disable TSX: it's proven to be an
@@ -2839,18 +2839,26 @@
 	}
 }
 
+/*
+ * As mentioned, we should only touch the MSR when we've got a suitable
+ * microcode loaded on this CPU.
+ */
 static void
-cpuid_apply_tsx(x86_taa_mitigation_t taa)
+cpuid_apply_tsx(x86_taa_mitigation_t taa, uchar_t *featureset)
 {
 	uint64_t val;
 
 	switch (taa) {
 	case X86_TAA_TSX_DISABLE:
+		if (!is_x86_feature(featureset, X86FSET_TSX_CTRL))
+			return;
 		val = rdmsr(MSR_IA32_TSX_CTRL);
 		val |= IA32_TSX_CTRL_CPUID_CLEAR | IA32_TSX_CTRL_RTM_DISABLE;
 		wrmsr(MSR_IA32_TSX_CTRL, val);
 		break;
 	case X86_TAA_TSX_FORCE_ABORT:
+		if (!is_x86_feature(featureset, X86FSET_TSX_CTRL))
+			return;
 		val = rdmsr(MSR_IA32_TSX_CTRL);
 		val |= IA32_TSX_CTRL_RTM_DISABLE;
 		wrmsr(MSR_IA32_TSX_CTRL, val);
@@ -2976,9 +2984,7 @@
 	 * Take care of certain mitigations on the non-boot CPU. The boot CPU
 	 * will have already run this function and determined what we need to
 	 * do. This gives us a hook for per-HW thread mitigations such as
-	 * enhanced IBRS, or disabling TSX.  For TSX disabling, we need to be
-	 * careful that we've had a chance to load ucode that enables the new
-	 * MSRs.
+	 * enhanced IBRS, or disabling TSX.
 	 */
 	if (cpu->cpu_id != 0) {
 		if (x86_spectrev2_mitigation == X86_SPECTREV2_ENHANCED_IBRS) {
@@ -2985,8 +2991,7 @@
 			cpuid_enable_enhanced_ibrs();
 		}
 
-		if (cpi->cpi_pass >= 1)
-			cpuid_apply_tsx(x86_taa_mitigation);
+		cpuid_apply_tsx(x86_taa_mitigation, featureset);
 		return;
 	}
 
@@ -3049,7 +3054,7 @@
 	 * also apply TAA mitigations.
 	 */
 	cpuid_update_tsx(cpu, featureset);
-	cpuid_apply_tsx(x86_taa_mitigation);
+	cpuid_apply_tsx(x86_taa_mitigation, featureset);
 }
 
 /*
--- old/usr/src/uts/i86pc/os/mp_startup.c	Mon Jan 13 09:14:47 2020
+++ new/usr/src/uts/i86pc/os/mp_startup.c	Mon Jan 13 09:14:47 2020
@@ -27,7 +27,7 @@
  * All rights reserved.
  */
 /*
- * Copyright 2019 Joyent, Inc.
+ * Copyright 2020 Joyent, Inc.
  * Copyright 2013 Nexenta Systems, Inc.  All rights reserved.
  * Copyright 2018 OmniOS Community Edition (OmniOSce) Association.
  */
@@ -1861,7 +1861,9 @@
 	(void) spl0();
 
 	/*
-	 * Fill out cpu_ucode_info.  Update microcode if necessary.
+	 * Fill out cpu_ucode_info.  Update microcode if necessary. Note that
+	 * this is done after pass1 on the boot CPU, but it needs to be later on
+	 * for the other CPUs.
 	 */
 	ucode_check(cp);
 	cpuid_pass_ucode(cp, new_x86_featureset);
